@@ -5,17 +5,19 @@ use std::fmt::{self, Debug};
 use std::hash::{BuildHasher, Hash};
 use std::mem::replace;
 use std::ops::{Deref, DerefMut, RangeInclusive};
-use std::pin::pin;
+#[cfg(not(feature = "loom"))]
 use std::sync::atomic::AtomicUsize;
 use std::sync::atomic::Ordering::Relaxed;
 
+#[cfg(feature = "loom")]
+use loom::sync::atomic::AtomicUsize;
 use sdd::{AtomicShared, Guard, Shared, Tag};
 
 use super::Equivalent;
+use super::hash_table::MAXIMUM_CAPACITY_LIMIT;
 use super::hash_table::bucket::{CACHE, DoublyLinkedList, EntryPtr};
 use super::hash_table::bucket_array::BucketArray;
 use super::hash_table::{HashTable, LockedBucket};
-use crate::async_helper::SendableGuard;
 
 /// Scalable concurrent 32-way associative cache backed by [`HashMap`](super::HashMap).
 ///
@@ -68,7 +70,7 @@ where
 {
     hashcache: &'h HashCache<K, V, H>,
     locked_bucket: LockedBucket<K, V, DoublyLinkedList, CACHE>,
-    entry_ptr: EntryPtr<'h, K, V, CACHE>,
+    entry_ptr: EntryPtr<K, V, CACHE>,
 }
 
 /// [`VacantEntry`] is a view into a vacant cache entry in a [`HashCache`].
@@ -84,15 +86,27 @@ where
 
 /// [`ConsumableEntry`] is a view into an occupied entry in a [`HashCache`] when iterating over
 /// entries in it.
-pub struct ConsumableEntry<'b, 'g: 'b, K, V> {
+pub struct ConsumableEntry<'b, K, V> {
     /// Holds an exclusive lock on the entry bucket.
     locked_bucket: &'b mut LockedBucket<K, V, DoublyLinkedList, CACHE>,
     /// Pointer to the entry.
-    entry_ptr: &'b mut EntryPtr<'g, K, V, CACHE>,
+    entry_ptr: &'b mut EntryPtr<K, V, CACHE>,
     /// Probes removal.
     remove_probe: &'b mut bool,
-    /// Associated [`Guard`].
-    guard: &'g Guard,
+}
+
+/// [`ReplaceResult`] is the result type of the [`HashCache::replace_async`] and
+/// [`HashCache::replace_sync`] methods.
+pub enum ReplaceResult<'h, K, V, H = RandomState>
+where
+    H: BuildHasher,
+{
+    /// The key was replaced.
+    Replaced(OccupiedEntry<'h, K, V, H>, K),
+    /// The key did not exist in the [`HashCache`].
+    ///
+    /// An [`OccupiedEntry`] can be created from the [`VacantEntry`].
+    NotReplaced(VacantEntry<'h, K, V, H>),
 }
 
 impl<K, V, H> HashCache<K, V, H>
@@ -136,7 +150,8 @@ where
 
     /// Creates an empty [`HashCache`] with the specified capacity and [`BuildHasher`].
     ///
-    /// The actual capacity is equal to or greater than the specified capacity.
+    /// The actual capacity is equal to or greater than `minimum_capacity` unless it is greater than
+    /// `1 << (usize::BITS - 1)`.
     ///
     /// # Examples
     ///
@@ -174,7 +189,7 @@ where
         let maximum_capacity = maximum_capacity
             .max(minimum_capacity.load(Relaxed))
             .max(BucketArray::<K, V, DoublyLinkedList, CACHE>::minimum_capacity())
-            .min(1_usize << (usize::BITS - 1))
+            .min(MAXIMUM_CAPACITY_LIMIT)
             .next_power_of_two();
         HashCache {
             bucket_array: array,
@@ -204,10 +219,8 @@ where
     #[inline]
     pub async fn entry_async(&self, key: K) -> Entry<'_, K, V, H> {
         let hash = self.hash(&key);
-        let sendable_guard = pin!(SendableGuard::default());
-        let locked_bucket = self.writer_async(hash, &sendable_guard).await;
-        let prolonged_guard = self.prolonged_guard_ref(sendable_guard.guard());
-        let entry_ptr = locked_bucket.search(&key, hash, prolonged_guard);
+        let locked_bucket = self.writer_async(hash).await;
+        let entry_ptr = locked_bucket.search(&key, hash);
         if entry_ptr.is_valid() {
             Entry::Occupied(OccupiedEntry {
                 hashcache: self,
@@ -245,10 +258,8 @@ where
     #[inline]
     pub fn entry_sync(&self, key: K) -> Entry<'_, K, V, H> {
         let hash = self.hash(&key);
-        let guard = Guard::new();
-        let prolonged_guard = self.prolonged_guard_ref(&guard);
-        let locked_bucket = self.writer_sync(hash, prolonged_guard);
-        let entry_ptr = locked_bucket.search(&key, hash, prolonged_guard);
+        let locked_bucket = self.writer_sync(hash);
+        let entry_ptr = locked_bucket.search(&key, hash);
         if entry_ptr.is_valid() {
             Entry::Occupied(OccupiedEntry {
                 hashcache: self,
@@ -284,9 +295,8 @@ where
     pub fn try_entry(&self, key: K) -> Option<Entry<'_, K, V, H>> {
         let hash = self.hash(&key);
         let guard = Guard::new();
-        let prolonged_guard = self.prolonged_guard_ref(&guard);
-        let locked_bucket = self.try_reserve_bucket(hash, prolonged_guard)?;
-        let entry_ptr = locked_bucket.search(&key, hash, prolonged_guard);
+        let locked_bucket = self.try_reserve_bucket(hash, &guard)?;
+        let entry_ptr = locked_bucket.search(&key, hash);
         if entry_ptr.is_valid() {
             Some(Entry::Occupied(OccupiedEntry {
                 hashcache: self,
@@ -326,14 +336,12 @@ where
     #[inline]
     pub async fn put_async(&self, key: K, val: V) -> Result<EvictedEntry<K, V>, (K, V)> {
         let hash = self.hash(&key);
-        let sendable_guard = pin!(SendableGuard::default());
-        let locked_bucket = self.writer_async(hash, &sendable_guard).await;
-        let guard = sendable_guard.guard();
-        if locked_bucket.search(&key, hash, guard).is_valid() {
+        let locked_bucket = self.writer_async(hash).await;
+        if locked_bucket.search(&key, hash).is_valid() {
             Err((key, val))
         } else {
             let evicted = locked_bucket.evict_lru_head(locked_bucket.data_block);
-            let entry_ptr = locked_bucket.insert(hash, (key, val), guard);
+            let entry_ptr = locked_bucket.insert(hash, (key, val));
             locked_bucket.update_lru_tail(&entry_ptr);
             Ok(evicted)
         }
@@ -364,187 +372,164 @@ where
     #[inline]
     pub fn put_sync(&self, key: K, val: V) -> Result<EvictedEntry<K, V>, (K, V)> {
         let hash = self.hash(&key);
-        let guard = Guard::new();
-        let locked_bucket = self.writer_sync(hash, &guard);
-        let entry_ptr = locked_bucket.search(&key, hash, &guard);
+        let locked_bucket = self.writer_sync(hash);
+        let entry_ptr = locked_bucket.search(&key, hash);
         if entry_ptr.is_valid() {
             Err((key, val))
         } else {
             let evicted = locked_bucket
                 .writer
                 .evict_lru_head(locked_bucket.data_block);
-            let entry_ptr = locked_bucket.insert(hash, (key, val), &guard);
+            let entry_ptr = locked_bucket.insert(hash, (key, val));
             locked_bucket.writer.update_lru_tail(&entry_ptr);
             Ok(evicted)
         }
     }
 
-    /// Gets an [`OccupiedEntry`] corresponding to the key for in-place modification.
-    ///
-    /// [`OccupiedEntry`] exclusively owns the entry, preventing others from gaining access to it:
-    /// use [`read_async`](Self::read_async) if read-only access is sufficient.
-    ///
-    /// Returns `None` if the key does not exist.
+    /// Adds a key to the [`HashCache`], replacing the existing key, if any, that is equal to the
+    /// given one.
     ///
     /// # Examples
     ///
     /// ```
-    /// use scc::HashCache;
+    /// use std::cmp::{Eq, PartialEq};
+    /// use std::hash::{Hash, Hasher};
     ///
-    /// let hashcache: HashCache<u64, u32> = HashCache::default();
-    /// let future_put = hashcache.put_async(11, 17);
-    /// let future_get = hashcache.get_async(&11);
+    /// use scc::HashCache;
+    /// use scc::hash_cache::ReplaceResult;
+    ///
+    /// #[derive(Debug)]
+    /// struct MaybeEqual(u64, u64);
+    ///
+    /// impl Eq for MaybeEqual {}
+    ///
+    /// impl Hash for MaybeEqual {
+    ///     fn hash<H: Hasher>(&self, state: &mut H) {
+    ///         // Do not read `self.1`.
+    ///         self.0.hash(state);
+    ///     }
+    /// }
+    ///
+    /// impl PartialEq for MaybeEqual {
+    ///     fn eq(&self, other: &Self) -> bool {
+    ///         // Do not compare `self.1`.
+    ///         self.0 == other.0
+    ///     }
+    /// }
+    ///
+    /// let hashcache: HashCache<MaybeEqual, usize> = HashCache::default();
+    ///
+    /// async {
+    ///     let ReplaceResult::NotReplaced(v) = hashcache.replace_async(MaybeEqual(11, 7)).await else {
+    ///         unreachable!();
+    ///     };
+    ///     drop(v.put_entry(17));
+    ///     let ReplaceResult::Replaced(_, k) = hashcache.replace_async(MaybeEqual(11, 11)).await else {
+    ///         unreachable!();
+    ///     };
+    ///     assert_eq!(k.1, 7);
+    /// };
     /// ```
     #[inline]
-    pub async fn get_async<Q>(&self, key: &Q) -> Option<OccupiedEntry<'_, K, V, H>>
-    where
-        Q: Equivalent<K> + Hash + ?Sized,
-    {
-        let hash = self.hash(key);
-        let sendable_guard = pin!(SendableGuard::default());
-        let locked_bucket = self.optional_writer_async(hash, &sendable_guard).await?;
-        let prolonged_guard = self.prolonged_guard_ref(sendable_guard.guard());
-        let entry_ptr = locked_bucket.search(key, hash, prolonged_guard);
+    pub async fn replace_async(&self, key: K) -> ReplaceResult<'_, K, V, H> {
+        let hash = self.hash(&key);
+        let locked_bucket = self.writer_async(hash).await;
+        let mut entry_ptr = locked_bucket.search(&key, hash);
         if entry_ptr.is_valid() {
-            locked_bucket.writer.update_lru_tail(&entry_ptr);
-            return Some(OccupiedEntry {
+            let prev_key = replace(
+                &mut entry_ptr
+                    .get_mut(locked_bucket.data_block, &locked_bucket.writer)
+                    .0,
+                key,
+            );
+            ReplaceResult::Replaced(
+                OccupiedEntry {
+                    hashcache: self,
+                    locked_bucket,
+                    entry_ptr,
+                },
+                prev_key,
+            )
+        } else {
+            ReplaceResult::NotReplaced(VacantEntry {
                 hashcache: self,
+                key,
+                hash,
                 locked_bucket,
-                entry_ptr,
-            });
+            })
         }
-        None
     }
 
-    /// Gets an [`OccupiedEntry`] corresponding to the key for in-place modification.
-    ///
-    /// [`OccupiedEntry`] exclusively owns the entry, preventing others from gaining access to it:
-    /// use [`read_sync`](Self::read_sync) if read-only access is sufficient.
-    ///
-    /// Returns `None` if the key does not exist.
+    /// Adds a key to the [`HashCache`], replacing the existing key, if any, that is equal to the
+    /// given one.
     ///
     /// # Examples
     ///
     /// ```
+    /// use std::cmp::{Eq, PartialEq};
+    /// use std::hash::{Hash, Hasher};
+    ///
     /// use scc::HashCache;
+    /// use scc::hash_cache::ReplaceResult;
     ///
-    /// let hashcache: HashCache<u64, u32> = HashCache::default();
+    /// #[derive(Debug)]
+    /// struct MaybeEqual(u64, u64);
     ///
-    /// assert!(hashcache.get_sync(&1).is_none());
-    /// assert!(hashcache.put_sync(1, 10).is_ok());
-    /// assert_eq!(*hashcache.get_sync(&1).unwrap().get(), 10);
+    /// impl Eq for MaybeEqual {}
     ///
-    /// *hashcache.get_sync(&1).unwrap() = 11;
-    /// assert_eq!(*hashcache.get_sync(&1).unwrap(), 11);
+    /// impl Hash for MaybeEqual {
+    ///     fn hash<H: Hasher>(&self, state: &mut H) {
+    ///         // Do not read `self.1`.
+    ///         self.0.hash(state);
+    ///     }
+    /// }
+    ///
+    /// impl PartialEq for MaybeEqual {
+    ///     fn eq(&self, other: &Self) -> bool {
+    ///         // Do not compare `self.1`.
+    ///         self.0 == other.0
+    ///     }
+    /// }
+    ///
+    /// let hashcache: HashCache<MaybeEqual, usize> = HashCache::default();
+    ///
+    /// let ReplaceResult::NotReplaced(v) = hashcache.replace_sync(MaybeEqual(11, 7)) else {
+    ///     unreachable!();
+    /// };
+    /// drop(v.put_entry(17));
+    /// let ReplaceResult::Replaced(_, k) = hashcache.replace_sync(MaybeEqual(11, 11)) else {
+    ///     unreachable!();
+    /// };
+    /// assert_eq!(k.1, 7);
     /// ```
     #[inline]
-    pub fn get_sync<Q>(&self, key: &Q) -> Option<OccupiedEntry<'_, K, V, H>>
-    where
-        Q: Equivalent<K> + Hash + ?Sized,
-    {
-        let hash = self.hash(key);
-        let guard = Guard::new();
-        let prolonged_guard = self.prolonged_guard_ref(&guard);
-        let locked_bucket = self.optional_writer_sync(hash, prolonged_guard)?;
-        let entry_ptr = locked_bucket.search(key, hash, prolonged_guard);
+    pub fn replace_sync(&self, key: K) -> ReplaceResult<'_, K, V, H> {
+        let hash = self.hash(&key);
+        let locked_bucket = self.writer_sync(hash);
+        let mut entry_ptr = locked_bucket.search(&key, hash);
         if entry_ptr.is_valid() {
-            locked_bucket.writer.update_lru_tail(&entry_ptr);
-            return Some(OccupiedEntry {
+            let prev_key = replace(
+                &mut entry_ptr
+                    .get_mut(locked_bucket.data_block, &locked_bucket.writer)
+                    .0,
+                key,
+            );
+            ReplaceResult::Replaced(
+                OccupiedEntry {
+                    hashcache: self,
+                    locked_bucket,
+                    entry_ptr,
+                },
+                prev_key,
+            )
+        } else {
+            ReplaceResult::NotReplaced(VacantEntry {
                 hashcache: self,
+                key,
+                hash,
                 locked_bucket,
-                entry_ptr,
-            });
+            })
         }
-        None
-    }
-
-    /// Reads a key-value pair.
-    ///
-    /// Returns `None` if the key does not exist.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use scc::HashCache;
-    ///
-    /// let hashcache: HashCache<u64, u32> = HashCache::default();
-    /// let future_put = hashcache.put_async(11, 17);
-    /// let future_read = hashcache.read_async(&11, |_, v| *v);
-    /// ```
-    #[inline]
-    pub async fn read_async<Q, R, F: FnOnce(&K, &V) -> R>(&self, key: &Q, reader: F) -> Option<R>
-    where
-        Q: Equivalent<K> + Hash + ?Sized,
-    {
-        let sendable_guard = pin!(SendableGuard::default());
-        let hash = self.hash(key);
-
-        self.reader_async(key, hash, reader, &sendable_guard).await
-    }
-
-    /// Reads a key-value pair.
-    ///
-    /// Returns `None` if the key does not exist.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use scc::HashCache;
-    ///
-    /// let hashcache: HashCache<u64, u32> = HashCache::default();
-    ///
-    /// assert!(hashcache.read_sync(&1, |_, v| *v).is_none());
-    /// assert!(hashcache.put_sync(1, 10).is_ok());
-    /// assert_eq!(hashcache.read_sync(&1, |_, v| *v).unwrap(), 10);
-    /// ```
-    #[inline]
-    pub fn read_sync<Q, R, F: FnOnce(&K, &V) -> R>(&self, key: &Q, reader: F) -> Option<R>
-    where
-        Q: Equivalent<K> + Hash + ?Sized,
-    {
-        let hash = self.hash(key);
-        let guard = Guard::new();
-        self.reader_sync(key, hash, reader, &guard)
-    }
-
-    /// Returns `true` if the [`HashCache`] contains a value for the specified key.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use scc::HashCache;
-    ///
-    /// let hashcache: HashCache<u64, u32> = HashCache::default();
-    ///
-    /// let future_contains = hashcache.contains_async(&1);
-    /// ```
-    #[inline]
-    pub async fn contains_async<Q>(&self, key: &Q) -> bool
-    where
-        Q: Equivalent<K> + Hash + ?Sized,
-    {
-        self.read_async(key, |_, _| ()).await.is_some()
-    }
-
-    /// Returns `true` if the [`HashCache`] contains a value for the specified key.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use scc::HashCache;
-    ///
-    /// let hashcache: HashCache<u64, u32> = HashCache::default();
-    ///
-    /// assert!(!hashcache.contains_sync(&1));
-    /// assert!(hashcache.put_sync(1, 0).is_ok());
-    /// assert!(hashcache.contains_sync(&1));
-    /// ```
-    #[inline]
-    pub fn contains_sync<Q>(&self, key: &Q) -> bool
-    where
-        Q: Equivalent<K> + Hash + ?Sized,
-    {
-        self.read_sync(key, |_, _| ()).is_some()
     }
 
     /// Removes a key-value pair if the key exists.
@@ -591,6 +576,165 @@ where
         self.remove_if_sync(key, |_| true)
     }
 
+    /// Gets an [`OccupiedEntry`] corresponding to the key for in-place modification.
+    ///
+    /// [`OccupiedEntry`] exclusively owns the entry, preventing others from gaining access to it:
+    /// use [`read_async`](Self::read_async) if read-only access is sufficient.
+    ///
+    /// Returns `None` if the key does not exist.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::HashCache;
+    ///
+    /// let hashcache: HashCache<u64, u32> = HashCache::default();
+    /// let future_put = hashcache.put_async(11, 17);
+    /// let future_get = hashcache.get_async(&11);
+    /// ```
+    #[inline]
+    pub async fn get_async<Q>(&self, key: &Q) -> Option<OccupiedEntry<'_, K, V, H>>
+    where
+        Q: Equivalent<K> + Hash + ?Sized,
+    {
+        let hash = self.hash(key);
+        let locked_bucket = self.optional_writer_async(hash).await?;
+        let entry_ptr = locked_bucket.search(key, hash);
+        if entry_ptr.is_valid() {
+            locked_bucket.writer.update_lru_tail(&entry_ptr);
+            return Some(OccupiedEntry {
+                hashcache: self,
+                locked_bucket,
+                entry_ptr,
+            });
+        }
+        None
+    }
+
+    /// Gets an [`OccupiedEntry`] corresponding to the key for in-place modification.
+    ///
+    /// [`OccupiedEntry`] exclusively owns the entry, preventing others from gaining access to it:
+    /// use [`read_sync`](Self::read_sync) if read-only access is sufficient.
+    ///
+    /// Returns `None` if the key does not exist.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::HashCache;
+    ///
+    /// let hashcache: HashCache<u64, u32> = HashCache::default();
+    ///
+    /// assert!(hashcache.get_sync(&1).is_none());
+    /// assert!(hashcache.put_sync(1, 10).is_ok());
+    /// assert_eq!(*hashcache.get_sync(&1).unwrap().get(), 10);
+    ///
+    /// *hashcache.get_sync(&1).unwrap() = 11;
+    /// assert_eq!(*hashcache.get_sync(&1).unwrap(), 11);
+    /// ```
+    #[inline]
+    pub fn get_sync<Q>(&self, key: &Q) -> Option<OccupiedEntry<'_, K, V, H>>
+    where
+        Q: Equivalent<K> + Hash + ?Sized,
+    {
+        let hash = self.hash(key);
+        let locked_bucket = self.optional_writer_sync(hash)?;
+        let entry_ptr = locked_bucket.search(key, hash);
+        if entry_ptr.is_valid() {
+            locked_bucket.writer.update_lru_tail(&entry_ptr);
+            return Some(OccupiedEntry {
+                hashcache: self,
+                locked_bucket,
+                entry_ptr,
+            });
+        }
+        None
+    }
+
+    /// Reads a key-value pair.
+    ///
+    /// Returns `None` if the key does not exist.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::HashCache;
+    ///
+    /// let hashcache: HashCache<u64, u32> = HashCache::default();
+    /// let future_put = hashcache.put_async(11, 17);
+    /// let future_read = hashcache.read_async(&11, |_, v| *v);
+    /// ```
+    #[inline]
+    pub async fn read_async<Q, R, F: FnOnce(&K, &V) -> R>(&self, key: &Q, reader: F) -> Option<R>
+    where
+        Q: Equivalent<K> + Hash + ?Sized,
+    {
+        self.reader_async(key, reader).await
+    }
+
+    /// Reads a key-value pair.
+    ///
+    /// Returns `None` if the key does not exist.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::HashCache;
+    ///
+    /// let hashcache: HashCache<u64, u32> = HashCache::default();
+    ///
+    /// assert!(hashcache.read_sync(&1, |_, v| *v).is_none());
+    /// assert!(hashcache.put_sync(1, 10).is_ok());
+    /// assert_eq!(hashcache.read_sync(&1, |_, v| *v).unwrap(), 10);
+    /// ```
+    #[inline]
+    pub fn read_sync<Q, R, F: FnOnce(&K, &V) -> R>(&self, key: &Q, reader: F) -> Option<R>
+    where
+        Q: Equivalent<K> + Hash + ?Sized,
+    {
+        self.reader_sync(key, reader)
+    }
+
+    /// Returns `true` if the [`HashCache`] contains a value for the specified key.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::HashCache;
+    ///
+    /// let hashcache: HashCache<u64, u32> = HashCache::default();
+    ///
+    /// let future_contains = hashcache.contains_async(&1);
+    /// ```
+    #[inline]
+    pub async fn contains_async<Q>(&self, key: &Q) -> bool
+    where
+        Q: Equivalent<K> + Hash + ?Sized,
+    {
+        self.reader_async(key, |_, _| ()).await.is_some()
+    }
+
+    /// Returns `true` if the [`HashCache`] contains a value for the specified key.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::HashCache;
+    ///
+    /// let hashcache: HashCache<u64, u32> = HashCache::default();
+    ///
+    /// assert!(!hashcache.contains_sync(&1));
+    /// assert!(hashcache.put_sync(1, 0).is_ok());
+    /// assert!(hashcache.contains_sync(&1));
+    /// ```
+    #[inline]
+    pub fn contains_sync<Q>(&self, key: &Q) -> bool
+    where
+        Q: Equivalent<K> + Hash + ?Sized,
+    {
+        self.read_sync(key, |_, _| ()).is_some()
+    }
+
     /// Removes a key-value pair if the key exists and the given condition is met.
     ///
     /// Returns `None` if the key does not exist or the condition was not met.
@@ -614,11 +758,10 @@ where
         Q: Equivalent<K> + Hash + ?Sized,
     {
         let hash = self.hash(key);
-        let sendable_guard = pin!(SendableGuard::default());
-        let mut locked_bucket = self.optional_writer_async(hash, &sendable_guard).await?;
-        let mut entry_ptr = locked_bucket.search(key, hash, sendable_guard.guard());
+        let mut locked_bucket = self.optional_writer_async(hash).await?;
+        let mut entry_ptr = locked_bucket.search(key, hash);
         if entry_ptr.is_valid() && condition(&mut locked_bucket.entry_mut(&mut entry_ptr).1) {
-            Some(locked_bucket.remove(self, &mut entry_ptr, sendable_guard.guard()))
+            Some(locked_bucket.remove(self, &mut entry_ptr))
         } else {
             None
         }
@@ -649,11 +792,10 @@ where
         Q: Equivalent<K> + Hash + ?Sized,
     {
         let hash = self.hash(key);
-        let guard = Guard::default();
-        let mut locked_bucket = self.optional_writer_sync(hash, &guard)?;
-        let mut entry_ptr = locked_bucket.search(key, hash, &guard);
+        let mut locked_bucket = self.optional_writer_sync(hash)?;
+        let mut entry_ptr = locked_bucket.search(key, hash);
         if entry_ptr.is_valid() && condition(&mut locked_bucket.entry_mut(&mut entry_ptr).1) {
-            Some(locked_bucket.remove(self, &mut entry_ptr, &guard))
+            Some(locked_bucket.remove(self, &mut entry_ptr))
         } else {
             None
         }
@@ -681,12 +823,10 @@ where
     /// ```
     #[inline]
     pub async fn iter_async<F: FnMut(&K, &V) -> bool>(&self, mut f: F) -> bool {
-        let sendable_guard = pin!(SendableGuard::default());
         let mut result = true;
-        self.for_each_reader_async(&sendable_guard, |reader, data_block| {
-            let guard = sendable_guard.guard();
-            let mut entry_ptr = EntryPtr::new(guard);
-            while entry_ptr.move_to_next(&reader, guard) {
+        self.for_each_reader_async(|reader, data_block| {
+            let mut entry_ptr = EntryPtr::null();
+            while entry_ptr.move_to_next(&reader) {
                 let (k, v) = entry_ptr.get(data_block);
                 if !f(k, v) {
                     result = false;
@@ -728,8 +868,8 @@ where
         let mut result = true;
         let guard = Guard::new();
         self.for_each_reader_sync(&guard, |reader, data_block| {
-            let mut entry_ptr = EntryPtr::new(&guard);
-            while entry_ptr.move_to_next(&reader, &guard) {
+            let mut entry_ptr = EntryPtr::null();
+            while entry_ptr.move_to_next(&reader) {
                 let (k, v) = entry_ptr.get(data_block);
                 if !f(k, v) {
                     result = false;
@@ -769,34 +909,24 @@ where
     /// };
     /// ```
     #[inline]
-    pub async fn iter_mut_async<F: FnMut(ConsumableEntry<'_, '_, K, V>) -> bool>(
+    pub async fn iter_mut_async<F: FnMut(ConsumableEntry<'_, K, V>) -> bool>(
         &self,
         mut f: F,
     ) -> bool {
-        let sendable_guard = pin!(SendableGuard::default());
         let mut result = true;
-        self.for_each_writer_async(0, 0, &sendable_guard, |mut locked_bucket| {
-            let mut removed = false;
-            let guard = sendable_guard.guard();
-            let mut entry_ptr = EntryPtr::new(guard);
-            while entry_ptr.move_to_next(&locked_bucket.writer, guard) {
+        self.for_each_writer_async(0, 0, |mut locked_bucket, removed| {
+            let mut entry_ptr = EntryPtr::null();
+            while entry_ptr.move_to_next(&locked_bucket.writer) {
                 let consumable_entry = ConsumableEntry {
                     locked_bucket: &mut locked_bucket,
                     entry_ptr: &mut entry_ptr,
-                    remove_probe: &mut removed,
-                    guard,
+                    remove_probe: removed,
                 };
                 if !f(consumable_entry) {
-                    if removed {
-                        locked_bucket.try_shrink_or_rebuild(self, guard);
-                    }
                     result = false;
                     return true;
                 }
             }
-            if removed {
-                locked_bucket.try_shrink_or_rebuild(self, guard);
-            }
             false
         })
         .await;
@@ -831,30 +961,22 @@ where
     /// assert_eq!(hashcache.len(), 2);
     /// ```
     #[inline]
-    pub fn iter_mut_sync<F: FnMut(ConsumableEntry<'_, '_, K, V>) -> bool>(&self, mut f: F) -> bool {
+    pub fn iter_mut_sync<F: FnMut(ConsumableEntry<'_, K, V>) -> bool>(&self, mut f: F) -> bool {
         let mut result = true;
         let guard = Guard::new();
-        self.for_each_writer_sync(0, 0, &guard, |mut locked_bucket| {
-            let mut removed = false;
-            let mut entry_ptr = EntryPtr::new(&guard);
-            while entry_ptr.move_to_next(&locked_bucket.writer, &guard) {
+        self.for_each_writer_sync(0, 0, &guard, |mut locked_bucket, removed| {
+            let mut entry_ptr = EntryPtr::null();
+            while entry_ptr.move_to_next(&locked_bucket.writer) {
                 let consumable_entry = ConsumableEntry {
                     locked_bucket: &mut locked_bucket,
                     entry_ptr: &mut entry_ptr,
-                    remove_probe: &mut removed,
-                    guard: &guard,
+                    remove_probe: removed,
                 };
                 if !f(consumable_entry) {
-                    if removed {
-                        locked_bucket.try_shrink_or_rebuild(self, &guard);
-                    }
                     result = false;
                     return true;
                 }
             }
-            if removed {
-                locked_bucket.try_shrink_or_rebuild(self, &guard);
-            }
             false
         });
         result
@@ -1059,9 +1181,8 @@ impl<K, V> HashCache<K, V, RandomState> {
 
     /// Creates an empty [`HashCache`] with the specified capacity.
     ///
-    /// The supplied minimum and maximum capacity values are adjusted to power-of-two values equal
-    /// to or larger than the provided values and `64` with one exception; if `0` is specified as
-    /// the minimum capacity, the minimum capacity of the `HashCache` becomes `0`.
+    /// The actual capacity is equal to or greater than `minimum_capacity` unless it is greater than
+    /// `1 << (usize::BITS - 1)`.
     ///
     /// # Examples
     ///
@@ -1180,12 +1301,12 @@ where
     }
 
     #[inline]
-    fn bucket_array(&self) -> &AtomicShared<BucketArray<K, V, DoublyLinkedList, CACHE>> {
+    fn bucket_array_var(&self) -> &AtomicShared<BucketArray<K, V, DoublyLinkedList, CACHE>> {
         &self.bucket_array
     }
 
     #[inline]
-    fn minimum_capacity(&self) -> &AtomicUsize {
+    fn minimum_capacity_var(&self) -> &AtomicUsize {
         &self.minimum_capacity
     }
 
@@ -1440,10 +1561,8 @@ where
     #[inline]
     #[must_use]
     pub fn remove_entry(mut self) -> (K, V) {
-        let guard = Guard::new();
-        let prolonged_guard = self.hashcache.prolonged_guard_ref(&guard);
         self.locked_bucket
-            .remove(self.hashcache, &mut self.entry_ptr, prolonged_guard)
+            .remove(self.hashcache, &mut self.entry_ptr)
     }
 
     /// Gets a reference to the value in the entry.
@@ -1636,15 +1755,11 @@ where
     /// ```
     #[inline]
     pub fn put_entry(self, val: V) -> (EvictedEntry<K, V>, OccupiedEntry<'h, K, V, H>) {
-        let guard = Guard::new();
-        let prolonged_guard = self.hashcache.prolonged_guard_ref(&guard);
         let evicted = self
             .locked_bucket
             .writer
             .evict_lru_head(self.locked_bucket.data_block);
-        let entry_ptr = self
-            .locked_bucket
-            .insert(self.hash, (self.key, val), prolonged_guard);
+        let entry_ptr = self.locked_bucket.insert(self.hash, (self.key, val));
         self.locked_bucket.writer.update_lru_tail(&entry_ptr);
         let occupied = OccupiedEntry {
             hashcache: self.hashcache,
@@ -1667,7 +1782,7 @@ where
     }
 }
 
-impl<K, V> ConsumableEntry<'_, '_, K, V> {
+impl<K, V> ConsumableEntry<'_, K, V> {
     /// Consumes the entry by moving out the key and value.
     ///
     /// # Examples
@@ -1699,11 +1814,11 @@ impl<K, V> ConsumableEntry<'_, '_, K, V> {
         *self.remove_probe |= true;
         self.locked_bucket
             .writer
-            .remove(self.locked_bucket.data_block, self.entry_ptr, self.guard)
+            .remove(self.locked_bucket.data_block, self.entry_ptr)
     }
 }
 
-impl<K, V> Deref for ConsumableEntry<'_, '_, K, V> {
+impl<K, V> Deref for ConsumableEntry<'_, K, V> {
     type Target = (K, V);
 
     #[inline]
@@ -1712,7 +1827,7 @@ impl<K, V> Deref for ConsumableEntry<'_, '_, K, V> {
     }
 }
 
-impl<K, V> DerefMut for ConsumableEntry<'_, '_, K, V> {
+impl<K, V> DerefMut for ConsumableEntry<'_, K, V> {
     #[inline]
     fn deref_mut(&mut self) -> &mut Self::Target {
         self.locked_bucket.entry_mut(self.entry_ptr)
