@@ -5,18 +5,18 @@ use std::fmt::{self, Debug};
 use std::hash::{BuildHasher, Hash};
 use std::mem::replace;
 use std::ops::{Deref, DerefMut, RangeInclusive};
-use std::pin::pin;
+#[cfg(not(feature = "loom"))]
 use std::sync::atomic::AtomicUsize;
-use std::sync::atomic::Ordering::{Acquire, Relaxed};
+use std::sync::atomic::Ordering::Relaxed;
 
+#[cfg(feature = "loom")]
+use loom::sync::atomic::AtomicUsize;
 use sdd::{AtomicShared, Guard, Shared, Tag};
 
 use super::Equivalent;
-use super::hash_table::HashTable;
 use super::hash_table::bucket::{EntryPtr, MAP};
 use super::hash_table::bucket_array::BucketArray;
-use crate::async_helper::SendableGuard;
-use crate::hash_table::LockedBucket;
+use super::hash_table::{HashTable, LockedBucket};
 
 /// Scalable concurrent hash map.
 ///
@@ -99,7 +99,7 @@ where
 {
     hashmap: &'h HashMap<K, V, H>,
     locked_bucket: LockedBucket<K, V, (), MAP>,
-    entry_ptr: EntryPtr<'h, K, V, MAP>,
+    entry_ptr: EntryPtr<K, V, MAP>,
 }
 
 /// [`VacantEntry`] is a view into a vacant entry in a [`HashMap`].
@@ -115,15 +115,27 @@ where
 
 /// [`ConsumableEntry`] is a view into an occupied entry in a [`HashMap`] when iterating over
 /// entries in it.
-pub struct ConsumableEntry<'b, 'g: 'b, K, V> {
+pub struct ConsumableEntry<'b, K, V> {
     /// Holds an exclusive lock on the entry bucket.
     locked_bucket: &'b mut LockedBucket<K, V, (), MAP>,
     /// Pointer to the entry.
-    entry_ptr: &'b mut EntryPtr<'g, K, V, MAP>,
+    entry_ptr: &'b mut EntryPtr<K, V, MAP>,
     /// Probes removal.
     remove_probe: &'b mut bool,
-    /// Associated [`Guard`].
-    guard: &'g Guard,
+}
+
+/// [`ReplaceResult`] is the result type of the [`HashMap::replace_async`] and
+/// [`HashMap::replace_sync`] methods.
+pub enum ReplaceResult<'h, K, V, H = RandomState>
+where
+    H: BuildHasher,
+{
+    /// The key was replaced.
+    Replaced(OccupiedEntry<'h, K, V, H>, K),
+    /// The key did not exist in the [`HashMap`].
+    ///
+    /// An [`OccupiedEntry`] can be created from the [`VacantEntry`].
+    NotReplaced(VacantEntry<'h, K, V, H>),
 }
 
 /// [`Reserve`] keeps the capacity of the associated [`HashMap`] higher than a certain level.
@@ -175,7 +187,8 @@ where
 
     /// Creates an empty [`HashMap`] with the specified capacity and [`BuildHasher`].
     ///
-    /// The actual capacity is equal to or greater than the specified capacity.
+    /// The actual capacity is equal to or greater than `capacity` unless it is greater than
+    /// `1 << (usize::BITS - 1)`.
     ///
     /// # Examples
     ///
@@ -281,10 +294,8 @@ where
     #[inline]
     pub async fn entry_async(&self, key: K) -> Entry<'_, K, V, H> {
         let hash = self.hash(&key);
-        let sendable_guard = pin!(SendableGuard::default());
-        let locked_bucket = self.writer_async(hash, &sendable_guard).await;
-        let prolonged_guard = self.prolonged_guard_ref(sendable_guard.guard());
-        let entry_ptr = locked_bucket.search(&key, hash, prolonged_guard);
+        let locked_bucket = self.writer_async(hash).await;
+        let entry_ptr = locked_bucket.search(&key, hash);
         if entry_ptr.is_valid() {
             Entry::Occupied(OccupiedEntry {
                 hashmap: self,
@@ -322,10 +333,8 @@ where
     #[inline]
     pub fn entry_sync(&self, key: K) -> Entry<'_, K, V, H> {
         let hash = self.hash(&key);
-        let guard = Guard::new();
-        let prolonged_guard = self.prolonged_guard_ref(&guard);
-        let locked_bucket = self.writer_sync(hash, prolonged_guard);
-        let entry_ptr = locked_bucket.search(&key, hash, prolonged_guard);
+        let locked_bucket = self.writer_sync(hash);
+        let entry_ptr = locked_bucket.search(&key, hash);
         if entry_ptr.is_valid() {
             Entry::Occupied(OccupiedEntry {
                 hashmap: self,
@@ -361,9 +370,8 @@ where
     pub fn try_entry(&self, key: K) -> Option<Entry<'_, K, V, H>> {
         let hash = self.hash(&key);
         let guard = Guard::new();
-        let prolonged_guard = self.prolonged_guard_ref(&guard);
-        let locked_bucket = self.try_reserve_bucket(hash, prolonged_guard)?;
-        let entry_ptr = locked_bucket.search(&key, hash, prolonged_guard);
+        let locked_bucket = self.try_reserve_bucket(hash, &guard)?;
+        let entry_ptr = locked_bucket.search(&key, hash);
         if entry_ptr.is_valid() {
             Some(Entry::Occupied(OccupiedEntry {
                 hashmap: self,
@@ -440,12 +448,10 @@ where
         &self,
         mut pred: P,
     ) -> Option<OccupiedEntry<'_, K, V, H>> {
-        let sendable_guard = pin!(SendableGuard::default());
         let mut entry = None;
-        self.for_each_writer_async(0, 0, &sendable_guard, |locked_bucket| {
-            let guard = self.prolonged_guard_ref(sendable_guard.guard());
-            let mut entry_ptr = EntryPtr::new(guard);
-            while entry_ptr.move_to_next(&locked_bucket.writer, guard) {
+        self.for_each_writer_async(0, 0, |locked_bucket, _| {
+            let mut entry_ptr = EntryPtr::null();
+            while entry_ptr.move_to_next(&locked_bucket.writer) {
                 let (k, v) = locked_bucket.entry(&entry_ptr);
                 if pred(k, v) {
                     entry = Some(OccupiedEntry {
@@ -484,10 +490,9 @@ where
     ) -> Option<OccupiedEntry<'_, K, V, H>> {
         let mut entry = None;
         let guard = Guard::new();
-        let prolonged_guard = self.prolonged_guard_ref(&guard);
-        self.for_each_writer_sync(0, 0, prolonged_guard, |locked_bucket| {
-            let mut entry_ptr = EntryPtr::new(prolonged_guard);
-            while entry_ptr.move_to_next(&locked_bucket.writer, prolonged_guard) {
+        self.for_each_writer_sync(0, 0, &guard, |locked_bucket, _| {
+            let mut entry_ptr = EntryPtr::null();
+            while entry_ptr.move_to_next(&locked_bucket.writer) {
                 let (k, v) = locked_bucket.entry(&entry_ptr);
                 if pred(k, v) {
                     entry = Some(OccupiedEntry {
@@ -525,13 +530,11 @@ where
     #[inline]
     pub async fn insert_async(&self, key: K, val: V) -> Result<(), (K, V)> {
         let hash = self.hash(&key);
-        let sendable_guard = pin!(SendableGuard::default());
-        let locked_bucket = self.writer_async(hash, &sendable_guard).await;
-        let guard = sendable_guard.guard();
-        if locked_bucket.search(&key, hash, guard).is_valid() {
+        let locked_bucket = self.writer_async(hash).await;
+        if locked_bucket.search(&key, hash).is_valid() {
             Err((key, val))
         } else {
-            locked_bucket.insert(hash, (key, val), guard);
+            locked_bucket.insert(hash, (key, val));
             Ok(())
         }
     }
@@ -560,12 +563,11 @@ where
     #[inline]
     pub fn insert_sync(&self, key: K, val: V) -> Result<(), (K, V)> {
         let hash = self.hash(&key);
-        let guard = Guard::new();
-        let locked_bucket = self.writer_sync(hash, &guard);
-        if locked_bucket.search(&key, hash, &guard).is_valid() {
+        let locked_bucket = self.writer_sync(hash);
+        if locked_bucket.search(&key, hash).is_valid() {
             Err((key, val))
         } else {
-            locked_bucket.insert(hash, (key, val), &guard);
+            locked_bucket.insert(hash, (key, val));
             Ok(())
         }
     }
@@ -640,10 +642,8 @@ where
         U: FnOnce(&K, &mut V) -> R,
     {
         let hash = self.hash(key);
-        let sendable_guard = pin!(SendableGuard::default());
-        let mut locked_bucket = self.optional_writer_async(hash, &sendable_guard).await?;
-        let guard = sendable_guard.guard();
-        let mut entry_ptr = locked_bucket.search(key, hash, guard);
+        let mut locked_bucket = self.optional_writer_async(hash).await?;
+        let mut entry_ptr = locked_bucket.search(key, hash);
         if entry_ptr.is_valid() {
             let (k, v) = locked_bucket.entry_mut(&mut entry_ptr);
             Some(updater(k, v))
@@ -675,9 +675,8 @@ where
         U: FnOnce(&K, &mut V) -> R,
     {
         let hash = self.hash(key);
-        let guard = Guard::default();
-        let mut locked_bucket = self.optional_writer_sync(hash, &guard)?;
-        let mut entry_ptr = locked_bucket.search(key, hash, &guard);
+        let mut locked_bucket = self.optional_writer_sync(hash)?;
+        let mut entry_ptr = locked_bucket.search(key, hash);
         if entry_ptr.is_valid() {
             let (k, v) = locked_bucket.entry_mut(&mut entry_ptr);
             Some(updater(k, v))
@@ -686,6 +685,152 @@ where
         }
     }
 
+    /// Adds a key to the [`HashMap`], replacing the existing key, if any, that is equal to the
+    /// given one.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use std::cmp::{Eq, PartialEq};
+    /// use std::hash::{Hash, Hasher};
+    ///
+    /// use scc::HashMap;
+    /// use scc::hash_map::ReplaceResult;
+    ///
+    /// #[derive(Debug)]
+    /// struct MaybeEqual(u64, u64);
+    ///
+    /// impl Eq for MaybeEqual {}
+    ///
+    /// impl Hash for MaybeEqual {
+    ///     fn hash<H: Hasher>(&self, state: &mut H) {
+    ///         // Do not read `self.1`.
+    ///         self.0.hash(state);
+    ///     }
+    /// }
+    ///
+    /// impl PartialEq for MaybeEqual {
+    ///     fn eq(&self, other: &Self) -> bool {
+    ///         // Do not compare `self.1`.
+    ///         self.0 == other.0
+    ///     }
+    /// }
+    ///
+    /// let hashmap: HashMap<MaybeEqual, usize> = HashMap::default();
+    ///
+    /// async {
+    ///     let ReplaceResult::NotReplaced(v) = hashmap.replace_async(MaybeEqual(11, 7)).await else {
+    ///         unreachable!();
+    ///     };
+    ///     drop(v.insert_entry(17));
+    ///     let ReplaceResult::Replaced(_, k) = hashmap.replace_async(MaybeEqual(11, 11)).await else {
+    ///         unreachable!();
+    ///     };
+    ///     assert_eq!(k.1, 7);
+    /// };
+    /// ```
+    #[inline]
+    pub async fn replace_async(&self, key: K) -> ReplaceResult<'_, K, V, H> {
+        let hash = self.hash(&key);
+        let locked_bucket = self.writer_async(hash).await;
+        let mut entry_ptr = locked_bucket.search(&key, hash);
+        if entry_ptr.is_valid() {
+            let prev_key = replace(
+                &mut entry_ptr
+                    .get_mut(locked_bucket.data_block, &locked_bucket.writer)
+                    .0,
+                key,
+            );
+            ReplaceResult::Replaced(
+                OccupiedEntry {
+                    hashmap: self,
+                    locked_bucket,
+                    entry_ptr,
+                },
+                prev_key,
+            )
+        } else {
+            ReplaceResult::NotReplaced(VacantEntry {
+                hashmap: self,
+                key,
+                hash,
+                locked_bucket,
+            })
+        }
+    }
+
+    /// Adds a key to the [`HashMap`], replacing the existing key, if any, that is equal to the
+    /// given one.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use std::cmp::{Eq, PartialEq};
+    /// use std::hash::{Hash, Hasher};
+    ///
+    /// use scc::HashMap;
+    /// use scc::hash_map::ReplaceResult;
+    ///
+    /// #[derive(Debug)]
+    /// struct MaybeEqual(u64, u64);
+    ///
+    /// impl Eq for MaybeEqual {}
+    ///
+    /// impl Hash for MaybeEqual {
+    ///     fn hash<H: Hasher>(&self, state: &mut H) {
+    ///         // Do not read `self.1`.
+    ///         self.0.hash(state);
+    ///     }
+    /// }
+    ///
+    /// impl PartialEq for MaybeEqual {
+    ///     fn eq(&self, other: &Self) -> bool {
+    ///         // Do not compare `self.1`.
+    ///         self.0 == other.0
+    ///     }
+    /// }
+    ///
+    /// let hashmap: HashMap<MaybeEqual, usize> = HashMap::default();
+    ///
+    /// let ReplaceResult::NotReplaced(v) = hashmap.replace_sync(MaybeEqual(11, 7)) else {
+    ///     unreachable!();
+    /// };
+    /// drop(v.insert_entry(17));
+    /// let ReplaceResult::Replaced(_, k) = hashmap.replace_sync(MaybeEqual(11, 11)) else {
+    ///     unreachable!();
+    /// };
+    /// assert_eq!(k.1, 7);
+    /// ```
+    #[inline]
+    pub fn replace_sync(&self, key: K) -> ReplaceResult<'_, K, V, H> {
+        let hash = self.hash(&key);
+        let locked_bucket = self.writer_sync(hash);
+        let mut entry_ptr = locked_bucket.search(&key, hash);
+        if entry_ptr.is_valid() {
+            let prev_key = replace(
+                &mut entry_ptr
+                    .get_mut(locked_bucket.data_block, &locked_bucket.writer)
+                    .0,
+                key,
+            );
+            ReplaceResult::Replaced(
+                OccupiedEntry {
+                    hashmap: self,
+                    locked_bucket,
+                    entry_ptr,
+                },
+                prev_key,
+            )
+        } else {
+            ReplaceResult::NotReplaced(VacantEntry {
+                hashmap: self,
+                key,
+                hash,
+                locked_bucket,
+            })
+        }
+    }
+
     /// Removes a key-value pair if the key exists.
     ///
     /// Returns `None` if the key does not exist.
@@ -752,12 +897,11 @@ where
     where
         Q: Equivalent<K> + Hash + ?Sized,
     {
-        let sendable_guard = pin!(SendableGuard::default());
         let hash = self.hash(key);
-        let mut locked_bucket = self.optional_writer_async(hash, &sendable_guard).await?;
-        let mut entry_ptr = locked_bucket.search(key, hash, sendable_guard.guard());
+        let mut locked_bucket = self.optional_writer_async(hash).await?;
+        let mut entry_ptr = locked_bucket.search(key, hash);
         if entry_ptr.is_valid() && condition(&mut locked_bucket.entry_mut(&mut entry_ptr).1) {
-            Some(locked_bucket.remove(self, &mut entry_ptr, sendable_guard.guard()))
+            Some(locked_bucket.remove(self, &mut entry_ptr))
         } else {
             None
         }
@@ -788,11 +932,10 @@ where
         Q: Equivalent<K> + Hash + ?Sized,
     {
         let hash = self.hash(key);
-        let guard = Guard::default();
-        let mut locked_bucket = self.optional_writer_sync(hash, &guard)?;
-        let mut entry_ptr = locked_bucket.search(key, hash, &guard);
+        let mut locked_bucket = self.optional_writer_sync(hash)?;
+        let mut entry_ptr = locked_bucket.search(key, hash);
         if entry_ptr.is_valid() && condition(&mut locked_bucket.entry_mut(&mut entry_ptr).1) {
-            Some(locked_bucket.remove(self, &mut entry_ptr, &guard))
+            Some(locked_bucket.remove(self, &mut entry_ptr))
         } else {
             None
         }
@@ -819,11 +962,9 @@ where
     where
         Q: Equivalent<K> + Hash + ?Sized,
     {
-        let sendable_guard = pin!(SendableGuard::default());
         let hash = self.hash(key);
-        let locked_bucket = self.optional_writer_async(hash, &sendable_guard).await?;
-        let guard = self.prolonged_guard_ref(sendable_guard.guard());
-        let entry_ptr = locked_bucket.search(key, hash, guard);
+        let locked_bucket = self.optional_writer_async(hash).await?;
+        let entry_ptr = locked_bucket.search(key, hash);
         if entry_ptr.is_valid() {
             return Some(OccupiedEntry {
                 hashmap: self,
@@ -861,10 +1002,8 @@ where
         Q: Equivalent<K> + Hash + ?Sized,
     {
         let hash = self.hash(key);
-        let guard = Guard::new();
-        let prolonged_guard = self.prolonged_guard_ref(&guard);
-        let locked_bucket = self.optional_writer_sync(hash, prolonged_guard)?;
-        let entry_ptr = locked_bucket.search(key, hash, prolonged_guard);
+        let locked_bucket = self.optional_writer_sync(hash)?;
+        let entry_ptr = locked_bucket.search(key, hash);
         if entry_ptr.is_valid() {
             return Some(OccupiedEntry {
                 hashmap: self,
@@ -893,9 +1032,7 @@ where
     where
         Q: Equivalent<K> + Hash + ?Sized,
     {
-        let sendable_guard = pin!(SendableGuard::default());
-        let hash = self.hash(key);
-        self.reader_async(key, hash, reader, &sendable_guard).await
+        self.reader_async(key, reader).await
     }
 
     /// Reads a key-value pair.
@@ -918,9 +1055,7 @@ where
     where
         Q: Equivalent<K> + Hash + ?Sized,
     {
-        let hash = self.hash(key);
-        let guard = Guard::new();
-        self.reader_sync(key, hash, reader, &guard)
+        self.reader_sync(key, reader)
     }
 
     /// Returns `true` if the [`HashMap`] contains a value for the specified key.
@@ -939,7 +1074,7 @@ where
     where
         Q: Equivalent<K> + Hash + ?Sized,
     {
-        self.read_async(key, |_, _| ()).await.is_some()
+        self.reader_async(key, |_, _| ()).await.is_some()
     }
 
     /// Returns `true` if the [`HashMap`] contains a value for the specified key.
@@ -985,12 +1120,10 @@ where
     /// ```
     #[inline]
     pub async fn iter_async<F: FnMut(&K, &V) -> bool>(&self, mut f: F) -> bool {
-        let sendable_guard = pin!(SendableGuard::default());
         let mut result = true;
-        self.for_each_reader_async(&sendable_guard, |reader, data_block| {
-            let guard = sendable_guard.guard();
-            let mut entry_ptr = EntryPtr::new(guard);
-            while entry_ptr.move_to_next(&reader, guard) {
+        self.for_each_reader_async(|reader, data_block| {
+            let mut entry_ptr = EntryPtr::null();
+            while entry_ptr.move_to_next(&reader) {
                 let (k, v) = entry_ptr.get(data_block);
                 if !f(k, v) {
                     result = false;
@@ -1032,8 +1165,8 @@ where
         let mut result = true;
         let guard = Guard::new();
         self.for_each_reader_sync(&guard, |reader, data_block| {
-            let mut entry_ptr = EntryPtr::new(&guard);
-            while entry_ptr.move_to_next(&reader, &guard) {
+            let mut entry_ptr = EntryPtr::null();
+            while entry_ptr.move_to_next(&reader) {
                 let (k, v) = entry_ptr.get(data_block);
                 if !f(k, v) {
                     result = false;
@@ -1073,34 +1206,24 @@ where
     /// };
     /// ```
     #[inline]
-    pub async fn iter_mut_async<F: FnMut(ConsumableEntry<'_, '_, K, V>) -> bool>(
+    pub async fn iter_mut_async<F: FnMut(ConsumableEntry<'_, K, V>) -> bool>(
         &self,
         mut f: F,
     ) -> bool {
-        let sendable_guard = pin!(SendableGuard::default());
         let mut result = true;
-        self.for_each_writer_async(0, 0, &sendable_guard, |mut locked_bucket| {
-            let mut removed = false;
-            let guard = sendable_guard.guard();
-            let mut entry_ptr = EntryPtr::new(guard);
-            while entry_ptr.move_to_next(&locked_bucket.writer, guard) {
+        self.for_each_writer_async(0, 0, |mut locked_bucket, removed| {
+            let mut entry_ptr = EntryPtr::null();
+            while entry_ptr.move_to_next(&locked_bucket.writer) {
                 let consumable_entry = ConsumableEntry {
                     locked_bucket: &mut locked_bucket,
                     entry_ptr: &mut entry_ptr,
-                    remove_probe: &mut removed,
-                    guard,
+                    remove_probe: removed,
                 };
                 if !f(consumable_entry) {
-                    if removed {
-                        locked_bucket.try_shrink_or_rebuild(self, guard);
-                    }
                     result = false;
                     return true;
                 }
             }
-            if removed {
-                locked_bucket.try_shrink_or_rebuild(self, guard);
-            }
             false
         })
         .await;
@@ -1135,30 +1258,22 @@ where
     /// assert_eq!(hashmap.len(), 2);
     /// ```
     #[inline]
-    pub fn iter_mut_sync<F: FnMut(ConsumableEntry<'_, '_, K, V>) -> bool>(&self, mut f: F) -> bool {
+    pub fn iter_mut_sync<F: FnMut(ConsumableEntry<'_, K, V>) -> bool>(&self, mut f: F) -> bool {
         let mut result = true;
         let guard = Guard::new();
-        self.for_each_writer_sync(0, 0, &guard, |mut locked_bucket| {
-            let mut removed = false;
-            let mut entry_ptr = EntryPtr::new(&guard);
-            while entry_ptr.move_to_next(&locked_bucket.writer, &guard) {
+        self.for_each_writer_sync(0, 0, &guard, |mut locked_bucket, removed| {
+            let mut entry_ptr = EntryPtr::null();
+            while entry_ptr.move_to_next(&locked_bucket.writer) {
                 let consumable_entry = ConsumableEntry {
                     locked_bucket: &mut locked_bucket,
                     entry_ptr: &mut entry_ptr,
-                    remove_probe: &mut removed,
-                    guard: &guard,
+                    remove_probe: removed,
                 };
                 if !f(consumable_entry) {
-                    if removed {
-                        locked_bucket.try_shrink_or_rebuild(self, &guard);
-                    }
                     result = false;
                     return true;
                 }
             }
-            if removed {
-                locked_bucket.try_shrink_or_rebuild(self, &guard);
-            }
             false
         });
         result
@@ -1335,10 +1450,10 @@ where
     ///
     /// let hashmap: HashMap<u64, u32> = HashMap::default();
     ///
-    /// assert_eq!(hashmap.capacity_range(), 0..=(1_usize << (usize::BITS - 1)));
+    /// assert_eq!(hashmap.capacity_range(), 0..=(1_usize << (usize::BITS - 2)));
     ///
     /// let reserved = hashmap.reserve(1000);
-    /// assert_eq!(hashmap.capacity_range(), 1000..=(1_usize << (usize::BITS - 1)));
+    /// assert_eq!(hashmap.capacity_range(), 1000..=(1_usize << (usize::BITS - 2)));
     /// ```
     #[inline]
     pub fn capacity_range(&self) -> RangeInclusive<usize> {
@@ -1390,7 +1505,8 @@ impl<K, V> HashMap<K, V, RandomState> {
 
     /// Creates an empty [`HashMap`] with the specified capacity.
     ///
-    /// The actual capacity is equal to or greater than the specified capacity.
+    /// The actual capacity is equal to or greater than `capacity` unless it is greater than
+    /// `1 << (usize::BITS - 1)`.
     ///
     /// # Examples
     ///
@@ -1518,19 +1634,14 @@ where
     }
 
     #[inline]
-    fn bucket_array(&self) -> &AtomicShared<BucketArray<K, V, (), MAP>> {
+    fn bucket_array_var(&self) -> &AtomicShared<BucketArray<K, V, (), MAP>> {
         &self.bucket_array
     }
 
     #[inline]
-    fn minimum_capacity(&self) -> &AtomicUsize {
+    fn minimum_capacity_var(&self) -> &AtomicUsize {
         &self.minimum_capacity
     }
-
-    #[inline]
-    fn maximum_capacity(&self) -> usize {
-        1_usize << (usize::BITS - 1)
-    }
 }
 
 impl<K, V, H> PartialEq for HashMap<K, V, H>
@@ -1775,10 +1886,7 @@ where
     #[inline]
     #[must_use]
     pub fn remove_entry(mut self) -> (K, V) {
-        let guard = Guard::new();
-        let prolonged_guard = self.hashmap.prolonged_guard_ref(&guard);
-        self.locked_bucket
-            .remove(self.hashmap, &mut self.entry_ptr, prolonged_guard)
+        self.locked_bucket.remove(self.hashmap, &mut self.entry_ptr)
     }
 
     /// Gets a reference to the value in the entry.
@@ -1895,17 +2003,11 @@ where
     pub async fn remove_and_async(self) -> ((K, V), Option<OccupiedEntry<'h, K, V, H>>) {
         let hashmap = self.hashmap;
         let mut entry_ptr = self.entry_ptr.clone();
-        let mut sendable_guard = pin!(SendableGuard::default());
-        let entry = self.locked_bucket.writer.remove(
-            self.locked_bucket.data_block,
-            &mut entry_ptr,
-            hashmap.prolonged_guard_ref(sendable_guard.guard()),
-        );
-        if let Some(locked_bucket) = self
+        let entry = self
             .locked_bucket
-            .next_async(hashmap, &mut entry_ptr, &mut sendable_guard)
-            .await
-        {
+            .writer
+            .remove(self.locked_bucket.data_block, &mut entry_ptr);
+        if let Some(locked_bucket) = self.locked_bucket.next_async(hashmap, &mut entry_ptr).await {
             return (
                 entry,
                 Some(OccupiedEntry {
@@ -1952,17 +2054,11 @@ where
     pub fn remove_and_sync(self) -> ((K, V), Option<Self>) {
         let hashmap = self.hashmap;
         let mut entry_ptr = self.entry_ptr.clone();
-        let guard = Guard::new();
-        let prolonged_guard = hashmap.prolonged_guard_ref(&guard);
-        let entry = self.locked_bucket.writer.remove(
-            self.locked_bucket.data_block,
-            &mut entry_ptr,
-            prolonged_guard,
-        );
-        if let Some(locked_bucket) =
-            self.locked_bucket
-                .next_sync(hashmap, &mut entry_ptr, prolonged_guard)
-        {
+        let entry = self
+            .locked_bucket
+            .writer
+            .remove(self.locked_bucket.data_block, &mut entry_ptr);
+        if let Some(locked_bucket) = self.locked_bucket.next_sync(hashmap, &mut entry_ptr) {
             return (
                 entry,
                 Some(OccupiedEntry {
@@ -1998,12 +2094,7 @@ where
     pub async fn next_async(self) -> Option<OccupiedEntry<'h, K, V, H>> {
         let hashmap = self.hashmap;
         let mut entry_ptr = self.entry_ptr.clone();
-        let mut sendable_guard = pin!(SendableGuard::default());
-        if let Some(locked_bucket) = self
-            .locked_bucket
-            .next_async(hashmap, &mut entry_ptr, &mut sendable_guard)
-            .await
-        {
+        if let Some(locked_bucket) = self.locked_bucket.next_async(hashmap, &mut entry_ptr).await {
             return Some(OccupiedEntry {
                 hashmap,
                 locked_bucket,
@@ -2043,12 +2134,7 @@ where
     pub fn next_sync(self) -> Option<Self> {
         let hashmap = self.hashmap;
         let mut entry_ptr = self.entry_ptr.clone();
-        let guard = Guard::new();
-        let prolonged_guard = hashmap.prolonged_guard_ref(&guard);
-        if let Some(locked_bucket) =
-            self.locked_bucket
-                .next_sync(hashmap, &mut entry_ptr, prolonged_guard)
-        {
+        if let Some(locked_bucket) = self.locked_bucket.next_sync(hashmap, &mut entry_ptr) {
             return Some(OccupiedEntry {
                 hashmap,
                 locked_bucket,
@@ -2155,11 +2241,7 @@ where
     /// ```
     #[inline]
     pub fn insert_entry(self, val: V) -> OccupiedEntry<'h, K, V, H> {
-        let guard = Guard::new();
-        let prolonged_guard = self.hashmap.prolonged_guard_ref(&guard);
-        let entry_ptr = self
-            .locked_bucket
-            .insert(self.hash, (self.key, val), prolonged_guard);
+        let entry_ptr = self.locked_bucket.insert(self.hash, (self.key, val));
         OccupiedEntry {
             hashmap: self.hashmap,
             locked_bucket: self.locked_bucket,
@@ -2180,7 +2262,7 @@ where
     }
 }
 
-impl<K, V> ConsumableEntry<'_, '_, K, V> {
+impl<K, V> ConsumableEntry<'_, K, V> {
     /// Consumes the entry by moving out the key and value.
     ///
     /// # Examples
@@ -2212,11 +2294,11 @@ impl<K, V> ConsumableEntry<'_, '_, K, V> {
         *self.remove_probe |= true;
         self.locked_bucket
             .writer
-            .remove(self.locked_bucket.data_block, self.entry_ptr, self.guard)
+            .remove(self.locked_bucket.data_block, self.entry_ptr)
     }
 }
 
-impl<K, V> Deref for ConsumableEntry<'_, '_, K, V> {
+impl<K, V> Deref for ConsumableEntry<'_, K, V> {
     type Target = (K, V);
 
     #[inline]
@@ -2225,7 +2307,7 @@ impl<K, V> Deref for ConsumableEntry<'_, '_, K, V> {
     }
 }
 
-impl<K, V> DerefMut for ConsumableEntry<'_, '_, K, V> {
+impl<K, V> DerefMut for ConsumableEntry<'_, K, V> {
     #[inline]
     fn deref_mut(&mut self) -> &mut Self::Target {
         self.locked_bucket.entry_mut(self.entry_ptr)
@@ -2294,8 +2376,8 @@ where
         debug_assert!(result >= self.additional);
 
         let guard = Guard::new();
-        if let Some(current_array) = self.hashmap.bucket_array.load(Acquire, &guard).as_ref() {
-            self.try_shrink_or_rebuild(current_array, 0, &guard);
+        if let Some(current_array) = self.hashmap.bucket_array(&guard) {
+            self.try_shrink(current_array, 0, &guard);
         }
     }
 }
