@@ -7,14 +7,10 @@ use std::fmt::{self, Debug};
 use std::hash::{BuildHasher, Hash};
 use std::mem::swap;
 use std::ops::{Deref, RangeInclusive};
-use std::pin::pin;
-
-use sdd::Guard;
 
+use super::hash_map;
 use super::hash_table::HashTable;
 use super::{Equivalent, HashMap};
-use crate::async_helper::SendableGuard;
-use crate::hash_map;
 
 /// Scalable concurrent hash set.
 ///
@@ -28,8 +24,8 @@ where
 
 /// [`ConsumableEntry`] is a view into an occupied entry in a [`HashSet`] when iterating over
 /// entries in it.
-pub struct ConsumableEntry<'w, 'g: 'w, K> {
-    consumable: hash_map::ConsumableEntry<'w, 'g, K, ()>,
+pub struct ConsumableEntry<'w, K> {
+    consumable: hash_map::ConsumableEntry<'w, K, ()>,
 }
 
 /// [`Reserve`] keeps the capacity of the associated [`HashSet`] higher than a certain level.
@@ -70,7 +66,8 @@ where
 
     /// Creates an empty [`HashSet`] with the specified capacity and [`BuildHasher`].
     ///
-    /// The actual capacity is equal to or greater than the specified capacity.
+    /// The actual capacity is equal to or greater than `capacity` unless it is greater than
+    /// `1 << (usize::BITS - 1)`.
     ///
     /// # Examples
     ///
@@ -219,16 +216,14 @@ where
     #[inline]
     pub async fn replace_async(&self, mut key: K) -> Option<K> {
         let hash = self.map.hash(&key);
-        let sendable_guard = pin!(SendableGuard::default());
-        let mut locked_bucket = self.map.writer_async(hash, &sendable_guard).await;
-        let guard = sendable_guard.guard();
-        let mut entry_ptr = locked_bucket.search(&key, hash, guard);
+        let mut locked_bucket = self.map.writer_async(hash).await;
+        let mut entry_ptr = locked_bucket.search(&key, hash);
         if entry_ptr.is_valid() {
             let k = &mut locked_bucket.entry_mut(&mut entry_ptr).0;
             swap(k, &mut key);
             Some(key)
         } else {
-            locked_bucket.insert(hash, (key, ()), guard);
+            locked_bucket.insert(hash, (key, ()));
             None
         }
     }
@@ -274,15 +269,14 @@ where
     #[inline]
     pub fn replace_sync(&self, mut key: K) -> Option<K> {
         let hash = self.map.hash(&key);
-        let guard = Guard::new();
-        let mut locked_bucket = self.map.writer_sync(hash, &guard);
-        let mut entry_ptr = locked_bucket.search(&key, hash, &guard);
+        let mut locked_bucket = self.map.writer_sync(hash);
+        let mut entry_ptr = locked_bucket.search(&key, hash);
         if entry_ptr.is_valid() {
             let k = &mut locked_bucket.entry_mut(&mut entry_ptr).0;
             swap(k, &mut key);
             Some(key)
         } else {
-            locked_bucket.insert(hash, (key, ()), &guard);
+            locked_bucket.insert(hash, (key, ()));
             None
         }
     }
@@ -549,10 +543,7 @@ where
     /// };
     /// ```
     #[inline]
-    pub async fn iter_mut_async<F: FnMut(ConsumableEntry<'_, '_, K>) -> bool>(
-        &self,
-        mut f: F,
-    ) -> bool {
+    pub async fn iter_mut_async<F: FnMut(ConsumableEntry<'_, K>) -> bool>(&self, mut f: F) -> bool {
         self.map
             .iter_mut_async(|consumable| f(ConsumableEntry { consumable }))
             .await
@@ -587,7 +578,7 @@ where
     /// assert_eq!(hashset.len(), 2);
     /// ```
     #[inline]
-    pub fn iter_mut_sync<F: FnMut(ConsumableEntry<'_, '_, K>) -> bool>(&self, mut f: F) -> bool {
+    pub fn iter_mut_sync<F: FnMut(ConsumableEntry<'_, K>) -> bool>(&self, mut f: F) -> bool {
         self.map
             .iter_mut_sync(|consumable| f(ConsumableEntry { consumable }))
     }
@@ -743,10 +734,10 @@ where
     ///
     /// let hashset: HashSet<u64> = HashSet::default();
     ///
-    /// assert_eq!(hashset.capacity_range(), 0..=(1_usize << (usize::BITS - 1)));
+    /// assert_eq!(hashset.capacity_range(), 0..=(1_usize << (usize::BITS - 2)));
     ///
     /// let reserved = hashset.reserve(1000);
-    /// assert_eq!(hashset.capacity_range(), 1000..=(1_usize << (usize::BITS - 1)));
+    /// assert_eq!(hashset.capacity_range(), 1000..=(1_usize << (usize::BITS - 2)));
     /// ```
     #[inline]
     pub fn capacity_range(&self) -> RangeInclusive<usize> {
@@ -827,7 +818,8 @@ impl<K: Eq + Hash> HashSet<K, RandomState> {
 
     /// Creates an empty [`HashSet`] with the specified capacity.
     ///
-    /// The actual capacity is equal to or greater than the specified capacity.
+    /// The actual capacity is equal to or greater than `capacity` unless it is greater than
+    /// `1 << (usize::BITS - 1)`.
     ///
     /// # Examples
     ///
@@ -913,7 +905,7 @@ where
     }
 }
 
-impl<K> ConsumableEntry<'_, '_, K> {
+impl<K> ConsumableEntry<'_, K> {
     /// Consumes the entry by moving out the key.
     ///
     /// # Examples
@@ -946,7 +938,7 @@ impl<K> ConsumableEntry<'_, '_, K> {
     }
 }
 
-impl<K> Deref for ConsumableEntry<'_, '_, K> {
+impl<K> Deref for ConsumableEntry<'_, K> {
     type Target = K;
 
     #[inline]
