@@ -1,8 +1,9 @@
 #![deny(missing_docs, warnings, clippy::all, clippy::pedantic)]
 #![doc = include_str!("../README.md")]
 
-pub mod bag;
-pub use bag::Bag;
+// Re-export [`sdd`](https://crates.io/crates/sdd) modules for backward compatibility.
+pub use sdd::{AtomicShared, Bag, Guard, LinkedEntry, LinkedList, Queue, Shared, Stack, Tag};
+pub use sdd::{bag, queue, stack};
 
 #[cfg(not(feature = "equivalent"))]
 mod equivalent;
@@ -20,18 +21,6 @@ pub use hash_map::HashMap;
 pub mod hash_set;
 pub use hash_set::HashSet;
 
-mod linked_list;
-pub use linked_list::Entry as LinkedEntry;
-pub use linked_list::LinkedList;
-
-pub mod queue;
-pub use queue::Queue;
-
-pub use sdd::{AtomicShared, Guard, Shared, Tag};
-
-pub mod stack;
-pub use stack::Stack;
-
 pub mod tree_index;
 pub use tree_index::TreeIndex;
 
